@@ -28,16 +28,16 @@ def find_objects(cls):
 
 
 def test_running():
-    from zeronimo.core import Component
-    class NullRunner(Component):
-        def run(self):
+    from zeronimo.core import Background
+    class NullBG(Background):
+        def __call__(self):
             gevent.sleep(0.1)
-    runner = NullRunner()
-    assert not runner.is_running()
-    runner.start()
-    assert runner.is_running()
-    runner.wait()
-    assert not runner.is_running()
+    bg = NullBG()
+    assert not bg.running()
+    bg.start()
+    assert bg.running()
+    bg.wait()
+    assert not bg.running()
 
 
 def test_messaging(ctx, addr, topic):
@@ -67,7 +67,7 @@ def test_from_socket(ctx, addr1, addr2):
     collector_sock.bind(addr2)
     push = ctx.socket(zmq.PUSH)
     push.connect(addr1)
-    # components
+    # logic
     app = Application()
     worker = zeronimo.Worker(app, [worker_sock])
     collector = zeronimo.Collector(collector_sock, addr2)
@@ -152,8 +152,8 @@ def test_fixtures(worker, push, pub, collector, addr1, addr2, ctx):
     assert isinstance(collector, zeronimo.Collector)
     assert addr1 != addr2
     assert isinstance(ctx, zmq.Context)
-    assert worker.is_running()
-    assert collector.is_running()
+    assert worker.running()
+    assert collector.running()
 
 
 def test_nowait(worker, push):
@@ -283,12 +283,11 @@ def test_reject(worker1, worker2, collector, push, pub, topic):
     # count accepted workers
     assert count_workers() == 2
     # worker1 uses a greenlet pool sized by 1
-    worker1.stop()
     worker1.greenlet_group = Pool(1)
-    worker1.start()
     # emit long task
-    assert len(fanout.emit(topic, 'sleep', 0.5)) == 2
-    assert len(fanout.emit(topic, 'sleep', 0.5)) == 1
+    how_slow = zeronimo.Fanout.timeout * 4
+    assert len(fanout.emit(topic, 'sleep', how_slow)) == 2
+    assert len(fanout.emit(topic, 'sleep', how_slow)) == 1
     assert count_workers() == 1
     # wait for long task done
     worker1.greenlet_group.wait_available()
@@ -297,22 +296,23 @@ def test_reject(worker1, worker2, collector, push, pub, topic):
 
 def test_subscription(worker1, worker2, collector, pub, topic):
     fanout = zeronimo.Fanout(pub, collector)
-    sub1 = [sock for sock in worker1.sockets if sock.socket_type == zmq.SUB][0]
-    sub2 = [sock for sock in worker2.sockets if sock.socket_type == zmq.SUB][0]
+    sub1 = [sock for sock in worker1.sockets if sock.type == zmq.SUB][0]
+    sub2 = [sock for sock in worker2.sockets if sock.type == zmq.SUB][0]
     sub1.set(zmq.UNSUBSCRIBE, topic)
     assert len(fanout.emit(topic, 'zeronimo')) == 1
     sub2.set(zmq.UNSUBSCRIBE, topic)
     assert fanout.emit(topic, 'zeronimo') == []
-    worker1.stop()
     sub1.set(zmq.SUBSCRIBE, topic)
-    sync_pubsub(pub, [sub1], topic)
-    worker1.start()
-    assert len(fanout.emit(topic, 'zeronimo')) == 1
-    worker1.stop()
+    # sync_pubsub will disturb the worker. so the worker should be stopped
+    # during sync_pubsub works.
     worker2.stop()
+    sync_pubsub(pub, [sub1], topic)
+    worker2.start()
+    assert len(fanout.emit(topic, 'zeronimo')) == 1
     sub2.set(zmq.SUBSCRIBE, topic)
+    # same reason
+    worker2.stop()
     sync_pubsub(pub, [sub2], topic)
-    worker1.start()
     worker2.start()
     assert len(fanout.emit(topic, 'zeronimo')) == 2
 
@@ -434,16 +434,18 @@ def test_proxied_collector(ctx, worker, push, addr1, addr2):
 
 
 def test_2nd_start(worker, collector):
-    assert worker.is_running()
+    assert worker.running()
+    print '- 1'
     worker.stop()
-    assert not worker.is_running()
+    assert not worker.running()
+    print '- 2'
     worker.start()
-    assert worker.is_running()
-    assert collector.is_running()
+    assert worker.running()
+    assert collector.running()
     collector.stop()
-    assert not collector.is_running()
+    assert not collector.running()
     collector.start()
-    assert collector.is_running()
+    assert collector.running()
 
 
 def test_concurrent_collector(worker, collector, push, pub, topic):
@@ -462,10 +464,10 @@ def test_concurrent_collector(worker, collector, push, pub, topic):
 def test_stopped_collector(worker, collector, push):
     customer = zeronimo.Customer(push, collector)
     collector.stop()
-    assert not collector.is_running()
+    assert not collector.running()
     result = customer.emit('zeronimo')
     assert result.get() == 'zeronimo'
-    assert collector.is_running()
+    assert collector.running()
     collector.stop()
 
 
@@ -578,7 +580,7 @@ def test_direct_xpub_xsub(ctx, addr1, addr2):
     collector_sock.bind(addr2)
     xpub = ctx.socket(zmq.XPUB)
     xpub.connect(addr1)
-    # components
+    # logic
     app = Application()
     worker = zeronimo.Worker(app, [worker_sock])
     collector = zeronimo.Collector(collector_sock, addr2)
@@ -611,7 +613,7 @@ def test_marshal_message(ctx, addr1, addr2):
     collector_sock = ctx.socket(zmq.PULL)
     collector_sock.bind(addr2)
     sockets = [worker_sock, collector_sock, customer_sock]
-    # components
+    # logic
     app = Application()
     worker = zeronimo.Worker(app, [worker_sock], pack=pack, unpack=unpack)
     collector = zeronimo.Collector(collector_sock, addr2, unpack=unpack)
@@ -637,6 +639,54 @@ def test_exception_handler(worker, collector, push, capsys):
     assert len(exceptions) == 1
 
 
+class ExampleException(BaseException):
+
+    errno = None
+    initialized = False
+
+    def __init__(self, errno):
+        self.errno = errno
+        self.initialized = True
+
+    def __getstate__(self):
+        return self.errno
+
+    def __setstate__(self, errno):
+        self.errno = errno
+
+
+class ExampleExceptionRaiser(object):
+
+    def throw(self, errno):
+        raise ExampleException(errno)
+
+
+def test_exception_state(ctx, addr1, addr2):
+    # sockets
+    worker_sock = ctx.socket(zmq.PULL)
+    worker_sock.bind(addr1)
+    collector_sock = ctx.socket(zmq.PULL)
+    collector_sock.bind(addr2)
+    push = ctx.socket(zmq.PUSH)
+    push.connect(addr1)
+    # logic
+    app = ExampleExceptionRaiser()
+    worker = zeronimo.Worker(app, [worker_sock])
+    collector = zeronimo.Collector(collector_sock, addr2)
+    customer = zeronimo.Customer(push, collector)
+    with running([worker], sockets=[worker_sock, collector_sock, push]):
+        result = customer.emit('throw', 2007)
+        try:
+            result.get()
+        except BaseException as exc:
+            assert isinstance(exc, zeronimo.RemoteException)
+            assert isinstance(exc, ExampleException)
+            assert exc.errno == 2007
+            assert not exc.initialized
+        else:
+            assert False
+
+
 # catch leaks
 
 
@@ -646,8 +696,9 @@ def test_no_worker_leak():
 
 
 @pytest.mark.trylast
-def test_no_customer_leak():
+def test_no_emitter_leak():
     assert not find_objects(zeronimo.Customer)
+    assert not find_objects(zeronimo.Fanout)
 
 
 @pytest.mark.trylast
